/*
 *  PlaySound - A MatLab MEX file to play a soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's hardware code
 *  dpwe 1994feb10
 *
 * 1997jun06 dpwe@icsi.berkeley.edu  Converted to Matlab 5
 * $Header: /n/crab/da/dpwe/src/SoundMex5/RCS/PlaySound.c,v 1.1 1997/06/12 00:53:28 dpwe Exp $
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include    "audIO.h"
#include    "mex.h"

void MyDouble2Short(register double *src, register short *dst, register long n)
{	/* do the format conversion */
	register double k = 32767.0;
	while(n--)
		*dst++ = k * *src++;
}

void
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    float   sampleRate;
    long    Length, channels;
    int     wordSize;
    int 	i, err;
    AUSTRUCT *aus;
    long   	pvl, pvb[16];
    short 	*sbuf = NULL;
    long 	sbsize = 32768;		/* frames written at once */

#ifdef DEBUG
    mexPrintf("PlaySound: Got %d lhs args and %d rhs args.\n", 
	       nlhs, nrhs);	
    for (i=0;i<nrhs;i++)  {
	mexPrintf("Arg #%d is size %d x %d\n", 
		   (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));
    }
    for (i=0;i<nlhs;i++)
	if (plhs[i])  {
	    mexPrintf("Arg #%d is size %d x %d\n", 
		       (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
	}
#endif				/* DEBUG */

    if (nrhs < 2){
	mexPrintf("PlaySound  PlaySound(D,SR[,B])  plays a sound on local hardware.\n");
	mexPrintf("           The data D should be scaled between -1.0 and 1.0,\n");
	mexPrintf("           with one row for each channel.  SR specifies the\n");
	mexPrintf("           sampling rate.  B, if present, specifies the buffer\n");
	mexPrintf("           size i.e. how many frames to convert at once (default:%ld).\n", sbsize);
	return;
    }
    /* look at the data */
    Length   = mxGetN(prhs[0]);
    channels = mxGetM(prhs[0]);
    if(channels > 16)  {		/* probably transposed - best not to write it */
	mexErrMsgTxt("There must be 16 or less channels (rows).");
	return;
    }
    sampleRate = mxGetScalar(prhs[1]);
    if(nrhs >= 3)  {		/* buffer size was specified */
	double specbuf = mxGetScalar(prhs[2]);
	if(specbuf > 1000)	/* ignore if zero or negative */
	    sbsize = specbuf;
    }

#ifdef DEBUG
    mexPrintf("PlaySound: %d chans, %ld frames, SR = %f\n", 
	       channels, Length, sampleRate); 
#endif				/* DEBUG */
    /* Open the audio port */
    if((aus=AUOpen(AFMT_SHORT, sampleRate, channels, sbsize/2, AIO_OUTPUT)) == NULL){
	mexErrMsgTxt("Problem initializing sound hardware.");
	return;
    }

    if( (sbuf = malloc(sbsize*channels*sizeof(short)))== NULL)  {
	AUClose(aus,1);
	mexErrMsgTxt("Could not allocate buffer for playing.");
	return;
    }
    {
	/* convert the doubles into buffer, play them out */
	long  todo, thistime;
	double *dsrc = mxGetPr(prhs[0]);

	todo = Length;
	while(todo)  {
	    thistime = (todo > sbsize)?sbsize:todo;

	    /*
	      ConvertSampleBlockFormat((char *)dsrc, (char *)sbuf, 
	      SFMT_DOUBLE, SFMT_SHORT, 
	      thistime*channels);
	      */
	    MyDouble2Short(dsrc, sbuf, thistime*channels);
	    AUWrite(aus, sbuf, thistime);
	    todo -= thistime;
	    dsrc += thistime*channels;
	}
#ifdef DEBUG
	mexPrintf("PlaySound: data written\n");
#endif				/* DEBUG */
    } 
    if(sbuf)
	free(sbuf);
#ifdef THINK_C
    {
	int blks, under;
	AUGetStats(aus, &blks, &under);
	mexPrintf("completed %d blocks, %d underruns\n", blks, under);
    }
#endif /* THINK_C */
    AUClose(aus,0);
#ifdef DEBUG
    mexPrintf("PlaySound: returning...\n");
#endif				/* DEBUG */
}

