/*
 *  WriteSound - A MatLab MEX file to write a soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's AIFF/C code etc.
 *  dpwe 1994feb10
 *
 * 1997jun06 dpwe@icsi.berkeley.edu  Updated for Matlab 5
 * $Header: /n/crab/da/dpwe/src/SoundMex5/RCS/WriteSound.c,v 1.1 1997/06/12 00:50:23 dpwe Exp $
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include    "snd.h"
#include    "mex.h"

#define     MAX_TEXT    300     /* Maximum size of string option */
#define     MAX_FMT	32	/* max size of format string */

#define     DFLT_FTYPE	"AIFF"

void
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    float   sampleRate;
    char    fileName[MAX_TEXT];
    long    Length, channels;
    int		outfmt = SFMT_SHORT;	/* always write these */
    int 	i;
    SOUND	*snd;
    long 	sbsize = 32768;		/* frames written at once */
    char	fmtName[MAX_FMT];

#ifdef DEBUG
    mexPrintf("WriteSound: Got %d lhs args and %d rhs args.\n", nlhs, nrhs);
    for (i=0;i<nrhs;i++)  {
	mexPrintf("Arg #%d is size %d x %d\n", 
		  (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));
    }
    for (i=0;i<nlhs;i++)
	if (plhs[i])  {
	    mexPrintf("Arg #%d is size %d x %d\n", 
		      (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
	}
#endif /* DEBUG */

    if (nrhs < 3){
	mexPrintf("WriteSound  WriteSound(FILE,D,SR,[,FMT])  writes a SNDF soundfile\n");
	mexPrintf("           to filename FILE containing the data in D which\n");
	mexPrintf("           should be scaled between -1.0 and 1.0, with one row\n");
	mexPrintf("           for each channel.  SR specifies the file's\n");
	mexPrintf("           sampling rate. FMT is a string defining the \n");
	mexPrintf("           output file type (default '" DFLT_FTYPE "', could be 'NIST', 'NeXT', \n");
	mexPrintf("           'MSWAVE', 'PCM/Eb', 'PCM/El', 'AIFF' - see sndf(3)) \n");
	return;
    }
    if (!mxIsChar(prhs[0])){
	mexErrMsgTxt("Argument 1 must be a string (filename).");
	return;
    } else {
	/* Convert to String */
	mxGetString(prhs[0], fileName, MAX_TEXT);
    }    
    if(nrhs > 3) {
	/* file type string is specified */
	if (!mxIsChar(prhs[3])){
	    mexErrMsgTxt("Argument 4 must be a string (filetype).");
	    return;
	}
	mxGetString(prhs[3], fmtName, MAX_FMT);
    } else {
	/* use default format name */
	strcpy(fmtName, DFLT_FTYPE);
    }
    /* look at the data */
    Length   = mxGetN(prhs[1]);
    channels = mxGetM(prhs[1]);
    if(channels > 16)  {	/* probably transposed - best not to write it */
	mexErrMsgTxt("There must be 16 or less channels (rows).");
	return;
    }
    sampleRate = mxGetScalar(prhs[2]);
#ifdef DEBUG
    mexPrintf("WriteSound: filename '%s', %ld chans, %ld frames, SR = %f\n", 
	      fileName, channels, Length, sampleRate);
#endif /* DEBUG */
    /* Set up the soundfile format */
    if(SFSetDefaultFormatByName(fmtName) == 0) {
	mexErrMsgTxt("unrecognized soundfile format name");
	return;
    }
    /* Open the soundfile */
    snd = sndNew(NULL);
    /* set up the params */
    sndSetSrate(snd, sampleRate);
    sndSetChans(snd, channels);
    sndSetFrames(snd, Length);
    sndSetFormat(snd, outfmt);
    /* but WE deal in doubles */
    sndSetUformat(snd, SFMT_DOUBLE);
	
    if( sndOpen(snd, fileName, "w") != NULL) {
	long  todo, thistime;
	double *dsrc = mxGetPr(prhs[1]);

	todo = Length;
	while(todo)  {
	    thistime = (todo > sbsize)?sbsize:todo;
	    sndWriteFrames(snd, dsrc, thistime);
	    todo -= thistime;
	    dsrc += thistime*channels;
	}
#ifdef DEBUG
	mexPrintf("WriteSound: data written\b");
#endif /* DEBUG */
	sndClose(snd);
    } else {
	mexErrMsgTxt("Could not open file.");
    }
    sndFree(snd);
#ifdef DEBUG
    mexPrintf("WriteSound: returning...\n");
#endif /* DEBUG */
}

